function Navigator(options,vectorLabels, vectorGrid, vectorNav, vectorLocation, vectorItem, mapMetersPerCoordinatePoint, data) {
	this.vectorLabels = vectorLabels;
	this.vectorGrid = vectorGrid;
	this.vectorNav = vectorNav;
	this.vectorLocation = vectorLocation;
	this.vectorItem = vectorItem;
	this.mapMetersPerCoordinatePoint = mapMetersPerCoordinatePoint;
	this.aproachingNode = options.aproachingNode;
	this.onPath = options.onPath;
	this.avarageWalkingSpeed = options.avarageWalkingSpeed;
	this.orientationTrigger = options.orientationTrigger;
    this.positioningMode = options.positioningMode;
	this.turnInstructionCrossingMinAngle = options.turnInstructionCrossingMinAngle;
	this.turnInstructionBendMinAngle = options.turnInstructionBendMinAngle
	this.lookRadius = options.lookRadius;
	this.navigatorCore = this;
	this.navBackboneJSON = appConnector.backboneJSON;
	this.routingTableJSON = appConnector.routingJSON;
	this.labelsJSON = appConnector.labelsJSON;
	this.arrivedDistance = options.arrivedDistance;
	this.approachingDistance = options.approachingDistance;
	this.reroutSkipCountMax = options.reroutSkipCountMax;
	this.reroutSkipCount=this.reroutSkipCountMax;

	this.initNavigator = function () {
		//console.log(this.labelsJSON);
		if(this.labelsJSON)
		this.loadLabels(this.labelsJSON);
		this.navBackboneFLayer = new NavBackboneFLayer(this.vectorGrid, this.navBackboneJSON);
		this.navBackboneFLayer.loadNavBackbone();
		this.navItemFLayer = new NavItemFLayer(this.vectorItem, data);
		this.navItemFLayer.loadNavItems();
		this.navItemFLayer.loadEvents();
		this.navPathAssistant = new NavPathAssistant(this.navigatorCore, this.vectorNav);
		this.tourPathAssistant = new TourPathAssistant(this.navigatorCore, this.vectorNav);
		this.loadRoutingTable();
		this.locationManager = new LocationManager(this.navigatorCore)
		this.locationManager.setPSType(this.positioningMode);
		this.userLocationDisplay = new UserLocationDisplay(this.navigatorCore, this.vectorLocation);
		this.orientationManager = new OrientationManager(this.navigatorCore)
		this.orientationManager.setAutoRotateMap(true);
		this.lookAssistant = new LookAssistant(this.navigatorCore);

	};

	this.loadRoutingTable = function () {
		this.routingTable = this.routingTableJSON;
		var points = this.navBackboneJSON['points'];
		var polygons = this.navBackboneJSON['polygons'];
		// resolve polygons
		for (var keyStart in points) {
			var startNode = points[keyStart];
			var polygon_id = startNode['polygon'];
			if (polygon_id) {
				var polygonData = polygons[polygon_id];
				var linkPoints = polygonData["linkPoints"];
				for (var pkey in linkPoints) {
					//console.log("pkey= "+ pkey);
					endNode = points[pkey];
					if (pkey != keyStart) {
					var	pointLinkData ={};
					//pointLinkData[endNode.id] = 'temp';
					if(!startNode["links"]) startNode["links"]={};
					startNode["links"][endNode.id] = 'temp';
					}
				}
				//console.log("graphNodeLinks pkey= "+ JSON.stringify(graphNodeLinks));
			}
		}
	};

	this.loadLabels = function (labelsJSON) {

		var labelsData = labelsJSON;
		var labelView;
		for (var key in labelsData) {
			//console.log("key= " + key);
			var labelData = labelsData[key];
			if (labelData.type == "text-label") {
				//console.log("labelData.type= "+ labelData.type)
				labelView = this.createPolygonLabel(labelData);
				this.vectorLabels.getSource().addFeature(labelView);

			} else if (labelData.type == "img-label") {
				//	console.log("labelData.type= "+ labelData.type)
				labelView = this.createIconLabel(labelData);
				this.vectorLabels.getSource().addFeature(labelView);

			}	if (labelData.type == "text-label-line") {
				//console.log("labelData.type= "+ labelData.type)
				labelView = this.createLineLabel(labelData);
				this.vectorLabels.getSource().addFeature(labelView);

			}

		}

	}

	this.createPolygonLabel = function(labelData) {
		var polygon = new ol.geom.Polygon(labelData['coord']);
		var nodeFeature =
			new ol.Feature({
				type : 'text-label',
				geometry : polygon,
				label : labelData['label'],
				labelIntl : labelData['labelIntl'],
			});
		return nodeFeature;
	}
	this.createLineLabel = function(labelData) {
		var line = new ol.geom.LineString(labelData['coord']);
		var nodeFeature =
			new ol.Feature({
				type : 'text-label-line',
				geometry : line,
				label : labelData['label'],
				labelIntl : labelData['labelIntl'],
			});
		return nodeFeature;
	}
	this.createIconLabel = function(labelData) {
		var iconLabel = new ol.Feature({
				type : 'img-label',
				geometry : new ol.geom.Point(labelData['coord']),
				labelImageURL : labelData['labelImageURL'],
				labelImageScale : labelData['labelImageScale']
			});
	iconLabel.setId( labelData['id'] );
		return iconLabel;
	}
	
	this.canLock = function (){
		if (this.locationManager.getLastKnownLocation()==undefined){
		return false;
		}
		//if (this.locationManager.psType == 'test'){
	//	 return true;
	//	}

		if( !ol.extent.containsCoordinate(viewableBounds, this.locationManager.getLastKnownLocation())){
				return false;
		}

	return true;
	}	


	this.userLocationChanged = function (newLocation, beaconData) {
	
		if(ol.extent.containsCoordinate(viewableBounds, newLocation)){
			this.orientationManager.setRotationCentre(newLocation);
		}
		this.userLocationDisplay.updateUserLocationDisplay(newLocation);

        if (lockMapOnUserLocation && this.canLock()) {
					//map.getView().animate({
					//center: newLocation,
					//duration: 100,
					//});
		map.getView().setCenter(this.locationManager.getLastKnownLocation());
		}
		this.lookAssistant.userLocationChanged(newLocation);
		
		if (this.reroutSkipCount<this.reroutSkipCountMax){
			this.reroutSkipCount++
			this.navPathAssistant.userLocationChanged(newLocation);
			return;
		}
		else {
			this.reroutSkipCount = 0;
			//this.userLocationDisplay.setDisplayUserLocation(false)
			//this.userLocationDisplay.updateUserLocationDisplay();
		}

		tourUserLocationChanged();

		if( this.navPathAssistant.arrived ) return;
		
			
		if(isTourMode()){
			reRouteTour();	
		} else {

		//	this.navPathAssistant.userLocationChanged(newLocation);
		//	
		  this.reRoute();
		}	
		 
	};
	
    this.reRoute = function () {
   	var currentDestID = navigatorCore.navPathAssistant.itemID
		if (currentDestID) {
			navigatorCore.navPathAssistant.clearNavigationAssistance()
			this.getDirectionsForTargetItem(currentDestID, true);
		}
   }
   
	this.deviceOrientationChanged = function (newDeviceRotation) {
		this.navPathAssistant.deviceOrientationChanged(newDeviceRotation);
		this.lookAssistant.deviceOrientationChanged(newDeviceRotation);
	};
	
	
	this.findAndDisplayTourPath = function (tourItems) {
		//console.log('getpathDirectionsForTargetItem: ' + target_id);
		

		var startLocationOptions;
		var destinationLocation;
		for (var key in tourItems) {
			var itemData = tourItems[key];
			
		//	console.log('getDirectionsForTour: ' + itemData['id']);
			if(!startLocationOptions) {
				startLocationOptions =  this.navItemFLayer.getNavBindingOptionsForItem(itemData['id']);
			}
			else  {
				if(destinationLocation){
					startLocationOptions= [destinationLocation]
				}
				
				var shortestPath = this.findShortestPathTour(startLocationOptions, itemData['id']) ;
				
				if(shortestPath ){
					//console.log('shortestPath: ' + JSON.stringify(shortestPath.routingData));
					destinationLocation = shortestPath['destinationLocation'];
					if(!destinationLocation){
						destinationLocation = shortestPath['end'];
						
					}
					//console.log('destinationLocation: ' + destinationLocation);
					this.tourPathAssistant.displayNavPath(shortestPath);
					
				} else {
					//appConnector.handleError('3',langString (language, 'warning_no_path'));
				}
			}
			
			
			
			
		}
	
	


	}
	

	this.getDirectionsForTargetItem = function (target_id, keepMode) {
		//console.log('getpathDirectionsForTargetItem: ' + target_id);
		$('#nav_error').hide();
		$("#navigation_overlay").show();
		if (!this.locationManager.getLastKnownLocation()){
			this.navPathAssistant.initForPath(undefined, target_id);
			appConnector.handleError('1',langString (language, 'warning_no_location'), true);
		return;
		}
		
		if(!ol.extent.containsCoordinate(viewableBounds, this.locationManager.getLastKnownLocation())){
				this.navPathAssistant.initForPath(undefined, target_id);
				appConnector.handleError('2',langString (language, 'warning_outside_bounds'), true);
				return;
		}
		
		var shortestPath = this.findShortestPath(target_id);
		if(shortestPath && shortestPath['routingData']){
			$('#nav-bar-navigation').show();
			$('#nav-bar-time').show();
			this.clearNavPathsCache();
			this.navPathAssistant.initForPath(shortestPath, target_id);
			this.displayNavPath(shortestPath);
			var userLocation = this.locationManager.getLastKnownLocation();
			this.navPathAssistant.userLocationChanged(userLocation);
			if(!keepMode)
			setMode(mode_lockAndRotateMap);	
		} else {
			this.navPathAssistant.initForPath(undefined, target_id);
			appConnector.handleError('3',langString (language, 'warning_no_path'), true);
		}

	}
	
	this.findDistanceToItem = function (itemID) {
		if (!this.locationManager.getLastKnownLocation())
			return;
		
		if(!ol.extent.containsCoordinate(viewableBounds, this.locationManager.getLastKnownLocation())){
			return;
		}
		if(Object.keys(this.navBackboneJSON).length < 1 ){
			return ;
		}
		var shortestPath = this.findShortestPath(itemID);
		//console.log("findDistanceToItem= "+JSON.stringify(shortestPath.d));
		if(!shortestPath)
			return;
	 
		var distanceToItem = (shortestPath.d +shortestPath.dtp) * this.mapMetersPerCoordinatePoint;
		
		return Math.round(distanceToItem);

	}
	
	this.findDistanceToItemM = function (itemID) {
		var d = this.findDistanceToItem(itemID) ;
		if (d) return d +langString(language,'metres');
		else return;

	}

	this.findShortestPath = function (target_id) {
		if (!this.locationManager.getLastKnownLocation())
			return;
		var userLocation = this.locationManager.getLastKnownLocation();
		var destBindingsOptions = this.navItemFLayer.getNavBindingOptionsForItem(target_id);
		//console.log("userLocation= "+userLocation);
		var startLocation = this.getSelectedLocation(userLocation);
		var shortestPath;
		for (var i = 0; i < destBindingsOptions.length; i++) {
			var destLocationOption = destBindingsOptions[i];
			var potentialShortestPath = this.findClosestPath(startLocation, destLocationOption);
			if (!shortestPath || shortestPath['d'] > potentialShortestPath['d'])
				//console.log("potentialShortestPath= "+potentialShortestPath);
				shortestPath = potentialShortestPath;
		}
		return shortestPath;
	}

	this.findShortestPathTour = function (startLocationOptions, target_id) {
         //console.log("findShortestPath= "+target_id);
		var destBindingsOptions = this.navItemFLayer.getNavBindingOptionsForItem(target_id);
		//console.log("userLocation= "+userLocation);

		//console.log("startLocation= "+JSON.stringify(startLocation));
		var shortestPath;
		for (var k = 0; k < startLocationOptions.length; k++) {
			for (var i = 0; i < destBindingsOptions.length; i++) {
				var destLocationOption = destBindingsOptions[i];
				var startLocation = startLocationOptions[k];
				var potentialShortestPath = this.findClosestPath(startLocation, destLocationOption);
				if (!shortestPath || shortestPath['d'] > potentialShortestPath['d']){
					shortestPath = potentialShortestPath;
					shortestPath['end'] = destLocationOption;
				}

			}
		}
		//shortestPath['itemCoord'] = destBindingsOptions['itemCoord'];

		return shortestPath;
	}
	
	this.findNavigationPath = function (startPoint, destPoint) {
		this.clearNavPathsCache();
		var startLocation = this.getSelectedLocation(startPoint);
		var destinationLocation = this.getSelectedLocation(destPoint);

		//var nodeStartClosest = getNavNode(start,selectedStartFeature);
		//console.log("startLocation['feature.type']= " + startLocation['feature'].get('type'));
		//console.log("startLocation['coord']= " + startLocation['coord']);

		var startLocationDisplayFeature = createCircleMarker(startLocation['coord'], "");
		this.vectorNav.getSource().addFeature(startLocationDisplayFeature);

		var destinationLocationDisplayFeature = createCircleMarker(destinationLocation['coord'], "temp", "");
		this.vectorNav.getSource().addFeature(destinationLocationDisplayFeature);

		var shortestPath = this.findClosestPath(startLocation, destinationLocation);
		this.displayNavPath(shortestPath);

	};

	this.getSelectedLocation = function (startPoint) {
		var lastKnownLocation = this.navigatorCore.locationManager.getLastKnownLocation();
	//	if(this.selectedLocationCache.startLocation && ( new Date().getTime() - this.selectedLocationCache.startLocationTime ) < 1000)
		if(this.selectedLocationCache.startLocation && lastKnownLocation && lastKnownLocation == this.selectedLocationCache.lastKnownLocation)
	    return 	this.selectedLocationCache.startLocation;
		
		var startLocation;
		var selectedStartFeature = this.navBackboneFLayer.getSelectedNavFeature(startPoint);
		if (!selectedStartFeature) {

			startLocation = this.navBackboneFLayer.findClosestFeatureAndPoint(startPoint);
			startLocation['startCoord'] = startPoint;
			//console.log("startLocation" + startLocation['feature'].get('type'));
		} else {
			startLocation = {};
			if (selectedStartFeature.get('type') == "nav-node-polygon") {
				startLocation['coord'] = startPoint
			} else {
				startLocation['coord'] = selectedStartFeature.getGeometry().getClosestPoint(startPoint);
			}
			startLocation['feature'] = selectedStartFeature;
		}
		this.selectedLocationCache.startLocation = startLocation;
		//this.selectedLocationCache.startLocationTime = new Date().getTime();
		this.selectedLocationCache.lastKnownLocation = lastKnownLocation;
		
		return startLocation;
	}
	
	
		this.getDestinationLocation = function (startPoint) {

		
		var startLocation;
		var selectedStartFeature = this.navBackboneFLayer.getSelectedNavFeature(startPoint);
		if (!selectedStartFeature) {

			startLocation = this.navBackboneFLayer.findClosestFeatureAndPoint(startPoint);
			startLocation['startCoord'] = startPoint;
			//console.log("startLocation" + startLocation['feature'].get('type'));
		} else {
			startLocation = {};
			if (selectedStartFeature.get('type') == "nav-node-polygon") {
				startLocation['coord'] = startPoint
			} else {
				startLocation['coord'] = selectedStartFeature.getGeometry().getClosestPoint(startPoint);
			}
			startLocation['feature'] = selectedStartFeature;
		}

		
		return startLocation;
	}
	
	this.selectedLocationCache={};
	
	this.getNavOptionsForNodes = function(locationClosestNodes ){
	//	console.log("getNavOptionsForNodes= " + JSON.stringify(locationClosestNodes));
		var locationClosestMultiLinkNodes = []	
		for (var skey in locationClosestNodes) {
			var startNodeID = skey;
		    var offsetStart = locationClosestNodes[skey];
			var startNode= this.navBackboneJSON.points[startNodeID];
			if (this.isMultiConnectNode (startNode)){
			//	console.log("isMultiConnectNode= " + startNodeID);
				var routingData = {};
				routingData['startNodeID'] = startNodeID;
				routingData['endNodeID'] = startNodeID;
				routingData['d'] = 0;
				routingData['p'] = [];
				routingData['offsetStart'] = offsetStart;
				this.registerMultiLinkNode(startNodeID,locationClosestMultiLinkNodes,routingData);

			}
			else {
			//	console.log("using path for " + startNodeID);
				var nodePaths = this.routingTable.pathMapping[startNodeID];
			//	console.log("nodePaths= " + JSON.stringify(nodePaths));
				
				for (var pKey in nodePaths) {
					var pathID = pKey;
					var path =  this.routingTable.pathRegister[pKey];
					//console.log("path= " + JSON.stringify(path));
					if (path.length > 0) {
						var hasMultiLinkNode = false;
						var nodeID = path[0];
						if (this.isMultiConnectNode (this.navBackboneJSON.points[nodeID])){
							this.registerMultiLinkNode(nodeID, locationClosestMultiLinkNodes, this.createRoutingData(startNodeID, nodeID, pathID, offsetStart));
							hasMultiLinkNode==true;
						}
						nodeID = path[path.length - 1];
						if (this.isMultiConnectNode (this.navBackboneJSON.points[nodeID])){
							this.registerMultiLinkNode(nodeID, locationClosestMultiLinkNodes, this.createRoutingData(startNodeID, nodeID, pathID, offsetStart));
							hasMultiLinkNode==true;
						}
						if(!hasMultiLinkNode){
							this.registerMultiLinkNode(nodeID, locationClosestMultiLinkNodes, this.createRoutingData(startNodeID, nodeID, pathID, offsetStart));
						}
						//var nodeIDEnd = path[path.length-1];
						
						//	if(nodeID==nodeIDEnd){
						//	this.registerMultiLinkNode(nodeID, locationClosestMultiLinkNodes, this.createRoutingData(startNodeID, nodeID, pathID, offsetStart));
						//}
					}

				}
			}
		
		}
		return locationClosestMultiLinkNodes;
	}

	this.registerMultiLinkNode = function (nodeID, locationClosestMultiLinkNodes, routingData) {
		//console.log("registerMultiLinkNode= " + nodeID + "" + this.isMultiConnectNode(this.navBackboneJSON.points[nodeID]));
		var existingEntry = locationClosestMultiLinkNodes[nodeID];
		//if (!existingEntry || (routingData.d + routingData.offsetStart) < (existingEntry.d + existingEntry.offsetStart))
			locationClosestMultiLinkNodes.push(routingData);
	}
		
	this.createRoutingData = function (startNodeID, endNodeID, pathID, offsetStart) {
		//console.log("createRoutingData= " + JSON.stringify(pathID));
		var path = this.routingTable.pathRegister[pathID];
		//console.log("createRoutingData path= " + JSON.stringify(path));
		var options = [];
		var iStartAll = this.indexOfAll(path, startNodeID)
			var iEndAll = this.indexOfAll(path, endNodeID)
			for (s = 0; s < iStartAll.length; s++) {
				for (e = 0; e < iEndAll.length; e++) {
					var routingData = {};
					routingData['startNodeID'] = startNodeID;
					routingData['endNodeID'] = endNodeID;
					routingData['d'] = 0;
					routingData['p'] = [];
					routingData['pathID'] = pathID;
					routingData['offsetStart'] = offsetStart;

					var iStart = iStartAll[s];
					var iEnd = iEndAll[e];
					//console.log("iStart= " + iStart);
					//console.log("iEnd= " + iEnd);
					var distance = 0;
					if (iStart <= iEnd) {
						var prevNode;
						for (i = iStart; i <= iEnd; i++) {
							routingData['p'].push(path[i]);
							var currentNode = this.navBackboneJSON.points[path[i]];
							//if(!currentNode) console.log(path[i]);
							if (prevNode) {
								routingData['d'] = routingData['d'] + this.getDistance(prevNode['coord'], currentNode['coord']);
							}
							prevNode = currentNode;
						}
					} else if (iStart >= iEnd) {
						var prevNode;
						for (i = iStart; i >= iEnd; i--) {
							routingData['p'].push(path[i]);
							var currentNode = this.navBackboneJSON.points[path[i]];
							//if(!currentNode) console.log(path[i]);
							if (prevNode) {
								routingData['d'] = routingData['d'] + this.getDistance(prevNode['coord'], currentNode['coord']);
							}
							prevNode = currentNode;
						}
					}

				}
				options.push(routingData);
			}
			var shortestD;
		var routingData;
		for (o = 0; o < options.length; o++) {
			if (!shortestD || options[o].d < shortestD) {
				shortestD = options[o].d;
				routingData = options[o];
			}

		}

		//console.log("createRoutingData= " + JSON.stringify(routingData));
		return routingData;
	}
	this.indexOfAll = function(array, searchItem) {
	  var i = array.indexOf(searchItem),
		  indexes = [];
		//  indexes.push(i);
	  while (i !== -1) {
		indexes.push(i);
		i = array.indexOf(searchItem, ++i);
	  }
	  
	  return indexes;
	}
	
	this.findClosestPath = function (startLocation, destinationLocation) {
		//console.log("findClosestPath= " + " s:" + JSON.stringify(startLocation.coord) + " d:" + JSON.stringify(destinationLocation.coord));

		var startLocationClosestNodes = this.navBackboneFLayer.getClosestNavNodesAndOffset(startLocation['coord'], startLocation['feature']);
		//console.log("startLocationClosestNodes= " + JSON.stringify(startLocationClosestNodes));
		var destinationLocationClosestNodes = this.navBackboneFLayer.getClosestNavNodesAndOffset(destinationLocation['coord'], destinationLocation['feature']);
		//console.log("destinationLocationClosestNodes= " + JSON.stringify(destinationLocationClosestNodes));

		var shortestPath = {};
		var distance = 0;
		if (startLocation.startCoord) {
			distance = distance + this.getDistance(startLocation['coord'], startLocation.startCoord);
		}

		var sharedNodeID;
		for (var skey in startLocationClosestNodes) {
			if (destinationLocationClosestNodes[skey] != undefined) {
				sharedNodeID = skey
					break;
			}
		}
		if (sharedNodeID && startLocation['feature'] != destinationLocation['feature']) {
			console.log("Shared Node");
			var routingData = {};
			routingData['p'] = [];
			routingData['d']=0;
			shortestPath['startLocation'] = startLocation;
			shortestPath['destinationLocation'] = destinationLocation;
			shortestPath['routingData'] = routingData;
			shortestPath['d'] = startLocationClosestNodes[sharedNodeID] + destinationLocationClosestNodes[sharedNodeID];

		} else if (startLocation['feature'] == destinationLocation['feature']) {
			//console.log("Same feature");
			var routingData = {};
			routingData['p'] = [];
			shortestPath['startLocation'] = startLocation;
			shortestPath['destinationLocation'] = destinationLocation;
			shortestPath['routingData'] = routingData;
			shortestPath['d'] = this.getDistance(startLocation['coord'], destinationLocation['coord']);
		} else {
			var startLocationClosestMultiLinkNodes = this.getNavOptionsForNodes(startLocationClosestNodes)
			var destLocationClosestMultiLinkNodes = this.getNavOptionsForNodes(destinationLocationClosestNodes)
			console.log("startNodeData all " + JSON.stringify(startLocationClosestMultiLinkNodes));
			console.log("destNodeData all  " + JSON.stringify(destLocationClosestMultiLinkNodes));
			for (var skey in startLocationClosestMultiLinkNodes) {
				var offsetStart;
				var offsetEnd;
				var startNodeData = startLocationClosestMultiLinkNodes[skey]
				var startNodeID = startNodeData.startNodeID;
				for (var dkey in destLocationClosestMultiLinkNodes) {
					var destNodeData = destLocationClosestMultiLinkNodes[dkey];
					var destNodeID = destNodeData.startNodeID; ;
					//console.log("startNodeData routingData " + JSON.stringify(startNodeData));
					//console.log("destNodeData routingData " + JSON.stringify(destNodeData));
					var routingData;
					//console.log("startNodeID == destNodeID " + startNodeID + '>' + destNodeID);
					if (startNodeID == destNodeID) {
						routingData = {};
						routingData['d'] = 0;
						routingData['p'] = [startNodeID];
						//console.log("startNodeID == destNodeID");
					} else {
						if (startNodeData && destNodeData) {
							var startPathID = startNodeData.pathID
								var destPathID = destNodeData.pathID
								if (startPathID && destPathID && startPathID == destPathID) {
									//console.log("On same path");
									offsetStart = startNodeData.offsetStart
									offsetEnd = destNodeData.offsetStart
									routingData = this.createRoutingData(startNodeID, destNodeID, startPathID, offsetStart)
									//console.log("routingData " + JSON.stringify(routingData));
								} else if (this.isMultiConnectNode(this.navBackboneJSON.points[startNodeData.endNodeID]) && this.isMultiConnectNode(this.navBackboneJSON.points[destNodeData.endNodeID]) ) {
									//console.log("Multilink nodes");
									offsetStart = startNodeData.offsetStart
									offsetEnd = destNodeData.offsetStart
									var rd;
									var routingDataMyultiLink = this.findRoutingPath(rd, startNodeData.endNodeID, destNodeData.endNodeID);
									if (routingDataMyultiLink) {
										routingData = {};
										routingData['p'] = [];
										var startPath;
										var endPath;
										if (startNodeData.p.length > 0) {
											startPath = startNodeData.p.slice(0, startNodeData.p.length - 1);
											Array.prototype.push.apply(routingData.p, startPath);
										}
										//console.log("routingDataMyultiLink " + JSON.stringify(routingDataMyultiLink));
										Array.prototype.push.apply(routingData.p, routingDataMyultiLink.p);
										if (destNodeData.p.length > 0) {
											endPath = destNodeData.p.slice(0, destNodeData.p.length - 1).reverse();
											Array.prototype.push.apply(routingData.p, endPath);
										}
										routingData['d'] = startNodeData.d + destNodeData.d + routingDataMyultiLink.d;
										//console.log("routingData final " + JSON.stringify(routingData));
									}
								}

						}

					}

					if (routingData) {
						var pathDistance = routingData['d'];
						var optionDistance = offsetStart + pathDistance + offsetEnd
						//console.log("shortestPath-routingData " + JSON.stringify(shortestPath['routingData']));
						//console.log("shortestPath-pathDistance " + pathDistance);
						if (!shortestPath['d'] || shortestPath['d'] > optionDistance) {
							shortestPath['d'] = optionDistance;
							shortestPath['routingData'] = routingData;
							//	console.log("shortestPath-routingData set" + JSON.stringify(shortestPath['routingData']));
							if (offsetStart > 0) {
								shortestPath['startLocation'] = startLocation;
							}
							if (offsetEnd > 0) {
								shortestPath['destinationLocation'] = destinationLocation;
							}
						}
					}
				}
			}
		}
		//console.log("shortestPath['routingData']" + JSON.stringify(shortestPath['routingData']));
		//console.log("shortestPath['d']" + JSON.stringify(shortestPath['d']));
		shortestPath['dtp'] = distance;
		return shortestPath;
	};
	
	this.isMultiConnectNode = function (node) {
		return node["links"] && Object.keys(node["links"]).length > 2;
	}
	this.isEndNode = function (node) {
		return node["links"] && Object.keys(node["links"]).length == 1;
	}
	
	this.findRoutingPath = function (routingData, startNodeID, destNodeID) {
		//if(routingData)
		//console.log("routingData " + JSON.stringify(routingData.p));
		//console.log("findRoutingPath start: " + startNodeID + " dest: " + destNodeID);
		var directRoutingData;
		if(this.routingTable[startNodeID]){
		//console.log("this.routingTable[startNodeID]" +  JSON.stringify(this.routingTable[startNodeID]));
			directRoutingData = this.routingTable[startNodeID][destNodeID];
		}

	     // console.log("directRoutingData" + directRoutingData);
		if (!directRoutingData && this.routingTable[destNodeID]) {
			var directRoutingDataInverted = this.routingTable[destNodeID][startNodeID];
			if (directRoutingDataInverted){
				directRoutingData ={};
				if(directRoutingDataInverted.p)
				directRoutingData.p = directRoutingDataInverted.p.slice().reverse();
				if(directRoutingDataInverted.r)
				directRoutingData.r = directRoutingDataInverted.r;
				directRoutingData.d = directRoutingDataInverted.d;
			}
		};
		//console.log("directRoutingData" + JSON.stringify(directRoutingData));
		if (!directRoutingData) return;
		
		
		if (directRoutingData.p) {
			if (routingData) {
				if(routingData.p.length>0) routingData.p.pop();
				Array.prototype.push.apply(routingData.p, directRoutingData.p);
				routingData.d = routingData.d + directRoutingData.d;
				return routingData;
			} else {
				return directRoutingData;
			}

		} else if (directRoutingData.r) {
			//console.log("directRoutingData.r " + directRoutingData.r);
			var pathFragment = this.getPathToIntersection(startNodeID, directRoutingData.r)

				if (!routingData) {
					routingData = {}
					routingData.p = [];
					routingData.d = 0
				}
			//console.log("routingData r " + JSON.stringify(pathFragment));
			if(pathFragment.p){
			if(routingData.p.length>0) routingData.p.pop();
			Array.prototype.push.apply(routingData.p, pathFragment.p);
		//	console.log("routingData r a " + JSON.stringify(routingData.p));
			routingData.d = routingData.d + pathFragment.d;
	    	return this.findRoutingPath(routingData, directRoutingData.r, destNodeID)
			}
			if(pathFragment.r){
			//console.log("routingData r r" + JSON.stringify(pathFragment.r));
			this.findRoutingPath(routingData, startNodeID, pathFragment.r)
		//	console.log("routingData r r a" + JSON.stringify(routingData.p));
	    	return this.findRoutingPath(routingData, pathFragment.r, destNodeID)
			}
		} 

	}

	this.getPathToIntersection = function (startNodeID, r) {
		var resolvedRoutingData = this.routingTable[startNodeID][r];
		if (!resolvedRoutingData && this.routingTable[r]) {
			var resolvedRoutingDataInverted = this.routingTable[r][startNodeID];
			if (resolvedRoutingDataInverted){
			
				resolvedRoutingData ={};
				if(resolvedRoutingDataInverted.p)
				resolvedRoutingData.p = resolvedRoutingDataInverted.p.slice().reverse();
				if(resolvedRoutingDataInverted.r)
				resolvedRoutingData.r = resolvedRoutingDataInverted.r	
				resolvedRoutingData.d = resolvedRoutingDataInverted.d
			}
				
		}
		return resolvedRoutingData;
	}

	
	
	this.createPathMap = function (shortestPath) {
		var pathNodes = [];
		var startLocation = shortestPath['startLocation'];
		if (startLocation) {
			var nodeData = {};
			nodeData['coord'] = startLocation['coord'];
			nodeData['nav_name'] = startLocation['feature']['nav_name'];
			nodeData['type'] = startLocation['feature'].get('type');
			nodeData['target_id'] = startLocation['feature'].getId();
			pathNodes.push(nodeData);
		}
		var navPath = shortestPath['routingData']['p'];
		for (var i = 0; i < navPath.length; i++) {
			var id = navPath[i];
			var navNodeData = this.navigatorCore.navBackboneFLayer.points[id];
			var nodeData = {};
			nodeData['coord'] = navNodeData['coord'];
			nodeData['nav_name'] = navNodeData['name'];
			nodeData['type'] = navNodeData['type'];
			nodeData['target_id'] = id;
			pathNodes.push(nodeData);
		}

		var destinationLocation = shortestPath['destinationLocation'];
		if (destinationLocation) {
			var nodeData = {};
			nodeData['coord'] = destinationLocation['coord'];
			nodeData['nav_name'] = destinationLocation['feature']['nav_name'];
			nodeData['type'] = destinationLocation['feature'].get('type');
			nodeData['target_id'] = destinationLocation['feature'].getId();
			pathNodes.push(nodeData);
		}
		if (destinationLocation && destinationLocation['itemData']) {
			var itemData = destinationLocation['itemData'];
			var nodeData = {};
			nodeData['coord'] = itemData['coord'];
			nodeData['nav_name'] = itemData['heading'];
			nodeData['type'] = itemData['type']
				nodeData['target_id'] = itemData['id'];
			pathNodes.push(nodeData);
		}
		return pathNodes;
	}

	this.displayNavPath = function (shortestPath) {

		var pathCoord = [];

		var startLocation = shortestPath['startLocation'];
		if (startLocation) {
			pathCoord.push(startLocation['coord']);
		}

		var navPath = shortestPath['routingData']['p'];
		for (var i = 0; i < navPath.length; i++) {
			var id = navPath[i];
			var node = this.navBackboneFLayer.points[id];
			pathCoord.push(node['coord']);
		}

		var destinationLocation = shortestPath['destinationLocation'];
		if (destinationLocation) {
			pathCoord.push(destinationLocation['coord']);
		}

		//	if (destinationLocation && destinationLocation['itemCoord']) {
		//	pathCoord.push(destinationLocation['itemCoord']);
		//	}

		var navPathFeature = this.createNavPathFeature(pathCoord);
		var navPathFeatureCore = this.createNavPathFeatureCore(pathCoord);

		this.vectorNav.getSource().addFeature(navPathFeature);
		this.vectorNav.getSource().addFeature(navPathFeatureCore);
		this.addArrowsToNavPath(navPathFeature);
	}

	this.clearNavPathsCache = function () {
		var features = this.vectorNav.getSource().getFeatures();
		for (var i = 0; i < features.length; i++) {
			if (features[i].get('purpose')!='tourPath')
			this.vectorNav.getSource().removeFeature(features[i]);
		}

	}

	this.navPathStyle = new ol.style.Style({ //  My style definition
			fill : new ol.style.Fill({
				color : 'rgba(217, 0, 0, 0.5)',
			}),
			stroke : new ol.style.Stroke({
				color : 'rgba(217, 0, 0, 0.5)',
				width : 18
				//lineCap: 'square',
				// lineDash: [20,40] // <-Here is the change. first line then space. try [40,40] if not shown as dash
			})
		});

	this.createNavPathFeature = function (pathCoord) {
		var linkFeature =
			new ol.Feature({
				id : 'linkFeature',
				type : 'nav-link-path',

				geometry : new ol.geom.LineString(pathCoord),
				name : 'link'
			});
		//nodeFeature.setId(id);
		linkFeature.setStyle(this.navPathStyle);
		//nodeFeature.setId(id);
		return linkFeature;

	}

	this.navPathCoreStyle = new ol.style.Style({ //  My style definition
			fill : new ol.style.Fill({
				//color : 'rgba(255, 204, 51, 1)'
				color : 'rgba(255, 255, 255, 1)'
			}),
			stroke : new ol.style.Stroke({
				//color : 'rgba(255, 204, 51, 1)',
				color : 'rgba(217, 255, 255, 1)',
				width : 2

				// lineCap: 'square',
				// lineDash: [20,40] // <-Here is the change. first line then space. try [40,40] if not shown as dash
			})
		});

	this.createNavPathFeatureCore = function (pathCoord) {
		var linkFeature =
			new ol.Feature({
				id : 'linkFeature',
				type : 'nav-link-core',

				geometry : new ol.geom.LineString(pathCoord),
				name : 'link'
			});
		//nodeFeature.setId(id);
		linkFeature.setStyle(this.navPathCoreStyle);
		//nodeFeature.setId(id);
		return linkFeature;

	}

	this.addArrowsToNavPath = function (feature) {
		var geometry = feature.getGeometry();
		geometry.forEachSegment(function (start, end) {
			var arrow = navigatorCore.addArrow(start, end);
			navigatorCore.vectorNav.getSource().addFeature(arrow);
		});
	};

	this.addArrow = function (start, end) {
		var x = start[0] + (end[0] - start[0]) / 2;
		var y = start[1] + (end[1] - start[1]) / 2;
		var arrow = new ol.Feature({
				id : 'arrow',
				type : 'icon',
				geometry : new ol.geom.Point([x, y]),
				name : 'arrow'
			});
		//arrow.setId('arrow');
		var dx = end[0] - start[0];
		var dy = end[1] - start[1];
		var rotation = Math.atan2(dy, dx);
		// arrows
		arrowStyle = new ol.style.Style({
				image : new ol.style.Icon({
					src : '././img/arrow-path.png',
					anchor : [0.75, 0.5],
					rotateWithView : true,
					rotation : -rotation,
					scale : 0.7
				})
			});
		arrow.setStyle(arrowStyle);
		return arrow;
	}

	this.getDistance = function (coord1, coord2) {
		return Math.sqrt(Math.pow(coord1[0] - coord2[0], 2) + Math.pow(coord1[1] - coord2[1], 2));
	}

}

function NavPathAssistant(navigatorCore, vectorNav) {
	this.vectorNav = vectorNav;
	this.navigatorCore = navigatorCore;
	this.isActive = false;
	this.pathFeatures = [];
	this.pathDirections = [];
	this.deviceOrientationDirections = [];
	this.currentDirections = {};
	this.arrived = false;
	this.approaching = false;
	
	this.initForPath = function (shortestPath, itemID) {

		this.itemID = itemID;
		showStaticMarkerOnMapTour(this.itemID, true)
		this.updateDestinatioinItemDetails(itemID);
		if (!shortestPath) {
			this.hasPath = false;
			//$("#btnModeResume").hide();
			$('#btnAR').hide();
			return;
		} else {	
			this.hasPath = true;
		}
		if (navigatorCore.navPathAssistant.isActive && navigatorCore.navPathAssistant.hasPath && lockMapOnUserLocation == false)
		$("#btnModeResume").show();
		
		if (mapViewerOptions.showAR) $('#btnAR').show();
		//console.log("initForPath" );
		this.isActive = true;
		this.pathFeatures = [];
		var pathNodes = [];
		var startLocation = shortestPath['startLocation'];
		if (startLocation) {
			var nodeData = {};
			nodeData['coord'] = startLocation['coord'];
			nodeData['nav_name'] = startLocation['feature']['nav_name'];
			nodeData['type'] = startLocation['feature'].get('type');
			nodeData['target_id'] = startLocation['feature'].getId();
			pathNodes.push(nodeData);
		}
			//	console.log(JSON.stringify(shortestPath))
		var navPath = shortestPath['routingData']['p'];

		for (var i = 0; i < navPath.length; i++) {
			var id = navPath[i];
			var navNodeData = this.navigatorCore.navBackboneFLayer.points[id];
			var nodeData = {};
			nodeData['coord'] = navNodeData['coord'];
			nodeData['nav_name'] = navNodeData['name'];
			nodeData['type'] = navNodeData['type'];
			nodeData['target_id'] = id;
			pathNodes.push(nodeData);
		}

		var destinationLocation = shortestPath['destinationLocation'];
		if (destinationLocation) {
			var nodeData = {};
			nodeData['coord'] = destinationLocation['coord'];
			nodeData['nav_name'] = destinationLocation['feature']['nav_name'];
			nodeData['type'] = destinationLocation['feature'].get('type');
			nodeData['target_id'] = destinationLocation['feature'].getId();
			pathNodes.push(nodeData);
		}


		// create invisible features
		var k = 0;
		for (var i = 1; i < pathNodes.length; i++) {
			var nodeDataStart = pathNodes[i - 1];
			if (i == 1) {
				var nodeDataStartFeature = this.createPathNodeFeature(nodeDataStart);
				this.pathFeatures.push(nodeDataStartFeature);
				nodeDataStart['feature'] = nodeDataStartFeature;
			}
			var nodeDataEnd = pathNodes[i];
			//calculate angles
			nodeDataStart['nextNode'] = nodeDataEnd;
			nodeDataEnd['prevNode'] = nodeDataStart;

			var pathSegmentLink = this.createPathSegmentFeature(nodeDataStart, nodeDataEnd);
			this.pathFeatures.push(pathSegmentLink);
			var nodeDataEndFeature = this.createPathNodeFeature(nodeDataEnd);
			this.pathFeatures.push(nodeDataEndFeature);
			nodeDataEnd['feature'] = nodeDataEndFeature;

		}
		for (var i = 0; i < pathNodes.length; i++) {
			this.calculateAngles(pathNodes[i]);
		}
		// show path
		//for (var i = 0; i < this.pathFeatures.length; i++) {

			//vectorNav.getSource().addFeature(this.pathFeatures[i]);

		//}
	//	if(this.pathFeatures && this.pathFeatures.length>0) {
	//		var destCoord = ol.proj.toLonLat((this.pathFeatures[this.pathFeatures.length-1].nodeData['coord']))
	//		this.updateDestinatioinItemDetails(itemID, destCoord);
	//	} else {
		//	this.updateDestinatioinItemDetails(itemID);
		//}
		
		if(this.pathFeatures && this.pathFeatures.length>0) {
				var destCoord = ol.proj.toLonLat((this.pathFeatures[this.pathFeatures.length-1].nodeData['coord']))
				this.currentDirections["lonLat"] = destCoord;
				//console.log('dest'+ JSON.stringify(destCoord))
		} 

	}

	this.updateDestinatioinItemDetails = function (itemID) {
		var itemData = this.navigatorCore.navItemFLayer.boundItemsData[itemID];	
		
	
		//var nextEvent = this.navigatorCore.navItemFLayer.getNextEvent(itemID);
		var nextEvent;
		if (itemData) {
			var isTourItem = tourItems && tourItems[itemID]!=undefined;
			if (isTourItem) {
				$('#tour-nav-info').show();		
		       $('#item-nav-info').hide();
			     $('#btnDismissNav').hide();

			} else {
	
				$('#tour-nav-info').hide();		
				$('#item-nav-info').show();
			}
		

			this.currentDirections["item_id"] = itemID
			this.currentDirections["item_name"] = itemData['popupHeading']
			this.currentDirections["item_preview_url"] = itemData['popupPreviewURL'];
			this.currentDirections["lonLat"] = itemData['lonLat'];
			var navPanelItemInfo = $('#item-nav-info');
			var navPanelItemEventInfo = $('#item-nav-event-info');
			if (navPanelItemInfo) {
				navPanelItemInfo.find('#item-nav-title').text(itemData['popupHeading']);
				navPanelItemInfo.find('#item-nav-img').attr('src', itemData['popupPreviewURL']);
				navPanelItemInfo.find('#item-nav-link').attr('target_id', itemData['id']);
				navPanelItemInfo.find('#tour-nav-link').attr('target_id', itemData['id']);
				//navPanelItemInfo.find('#item-nav-link').attr('data-target', getItemTargetModal(itemID));
				navPanelItemInfo.find('#item-nav-ar').attr('href',"app://arView?lonLat="+itemData["lonLat"]);
				if (nextEvent) {
					navPanelItemInfo.find('#item-nav-event-type').text(nextEvent.type);
					var timeTo = displayTimeFromSecondsToHoursAndMinutes(nextEvent.start_time_sec - getSecondsFromDayStart(new Date()))
						navPanelItemInfo.find('#item-nav-event-timeto').text('in ' + timeTo);
					navPanelItemInfo.find('#item-nav-event-time').text(nextEvent.start_time);
					navPanelItemInfo.find('#item-nav-event-status').text(nextEvent.status);
					navPanelItemEventInfo.show();
				} else {

					navPanelItemEventInfo.hide();
				}
				//	navPanelItemInfo.find('#item-modal-link-dir').attr('target-id', targer_id);

				//    <div id="item-nav-event-info" class="item-nav-event-info">
				//	<span id="item-nav-event-type" class="item-nav-event-type">Talk</span>
				//	<span id="item-nav-event-time" class="item-nav-event-time">in 16 min (3:00 pm)</span>
				//	<span id="item-nav-event-status" class="item-nav-event-status">Open</span>
				//navPanelItemInfo.show();
			}
		}

	}

	this.removeDestinatioinItemDetails = function () {
		var navPanelItemInfo = $('#item-nav-info')
			if (navPanelItemInfo) {
				navPanelItemInfo.find('#item-nav-title').text('');
				navPanelItemInfo.find('#item-nav-img').attr('src', '');
				//navPanelItemInfo.hide();
			}

	}

	this.stopNavigationAssistance = function () {
		if (mapViewerOptions.showAR) $('#btnAR').hide();
		this.navigatorCore.clearNavPathsCache();
		this.pathFeatures = [];
		this.pathDirections = [];
		this.deviceOrientationDirections = [];
		this.isActive = false;
		this.arrived = false;
		this.approaching = false;
		this.itemID=undefined;
		this.removeDestinatioinItemDetails();
		popupNav.hide();
		$('#tour_arrived_overlay').hide();
		$("#navigation_overlay").hide();
		//$("#btnModeResume").hide();
		$('#tour_arrived_overlay').empty();
	}
	
	this.clearNavigationAssistance = function () {
		this.navigatorCore.clearNavPathsCache();
		this.pathFeatures = [];
		this.pathDirections = [];
		this.deviceOrientationDirections = [];
	}
	
	this.updateArrivedOverlay = function () {
		if (!this.itemID)
			return;
		var itemData = navigatorCore.navItemFLayer.boundItemsData[this.itemID];
		if (this.approaching || this.arrived) {
			if (isTourMode()) {
				initArrivedOverlayTour(itemData.id)
				updateArrivedOverlayTour(itemData.id, navigatorCore.navPathAssistant.arrived);
			} else {
				initArrivedOverlay(itemData.id)
				updateArrivedOverlay(itemData.id, navigatorCore.navPathAssistant.arrived);
			}
			$('#tour_arrived_overlay').show();
		} else {
			$('#tour_arrived_overlay').hide();
		}

	}
	
	this.setArrived = function (flag) {
		this.arrived = flag
			if (this.arrived) {
				this.clearNavigationAssistance()
				//$("#btnModeResume").hide();
				$('#btnAR').hide();
				if (tourItems && tourItems[this.itemID] && tourItems[this.itemID]['s'] == 0) {
					// register as visited
					registerAsVisisted(this.itemID)
				}
			}

	}
	this.setApproaching = function (flag) {
		this.approaching = flag
	}
	
	this.createPathNodeFeature = function (nodeData) {
		var point = new ol.geom.Point(nodeData['coord']);
		var nodeFeature =
			new ol.Feature({
				type : 'path-node',
				geometry : point,
			});
		nodeFeature['nodeData'] = nodeData;
		nodeFeature.setStyle(this.nodeStyleInvisible);
		return nodeFeature;
	}
	this.createPathNodeFeatureHL = function (nodeData) {
		var point = new ol.geom.Point(nodeData['coord']);
		var nodeFeature =
			new ol.Feature({
				type : 'path-node',
				geometry : point,
			});
		nodeFeature['nodeData'] = nodeData;
		nodeFeature.setStyle(this.nodeStyleHL);
		return nodeFeature;
	}
	this.nodeStyleHL = new ol.style.Style({
			image : new ol.style.Circle({
				radius : 9,
				fill : new ol.style.Fill({
					color : 'rgba(0, 255, 0, 1)'
				}),
				stroke : new ol.style.Stroke({
					color : 'rgba(0, 255, 0, 1)',
					width : 2
				})
			})
		});
	this.nodeStyleInvisible = new ol.style.Style({
			image : new ol.style.Circle({
				radius : 7,
				fill : new ol.style.Fill({
					color : 'rgba(255, 0, 0, 0)'
				}),
				stroke : new ol.style.Stroke({
					color : 'rgba(255, 0, 0, 0)',
					width : 2
				})
			})
		});

	this.linkStyleInvisible = new ol.style.Style({
			stroke : new ol.style.Stroke({
				color : 'rgba(255, 0, 0, 0)',
				width : 1

			})
		});

	this.createPathSegmentFeature = function (nodeDataStart, nodeDataEnd) {
		//console.log("nodeDataStart " + nodeDataStart['coord']);
		//console.log("nodeDataEnd " + nodeDataEnd['coord']);
		var linkFeature =
			new ol.Feature({
				//id:   'linkFeature',
				type : 'path-link',
				geometry : new ol.geom.LineString([nodeDataStart['coord'], nodeDataEnd['coord']]),
				name : 'link'
			});
		//nodeFeature.setId(startNode.getId());
		linkFeature['nodeDataStart'] = nodeDataStart;
		linkFeature['nodeDataEnd'] = nodeDataEnd;
		linkFeature.setStyle(this.linkStyleInvisible);
		return linkFeature;

	}

	this.calculateAngles = function (nodeData) {
		// get angle relative to axis Y
		//angle(v1, v2) = acos( (v1x * v2x + v1y * v2y) / (sqrt(v1x^2+v1y^2) * sqrt(v2x^2+v2y^2)) )

		var nextNode = nodeData['nextNode'];
		if (nextNode) {
			// get angle to axis y;

			var v1 = {
				'x' : 0,
				'y' : 100
			}
			var v2 = {
				'x' : nextNode['coord'][0] - nodeData['coord'][0],
				'y' : nextNode['coord'][1] - nodeData['coord'][0]

			}

			//var result = this.calculateAngle2Vectors(v1,v2);
			var results = this.calculateAngle2Vectors2(nodeData['coord'], nextNode['coord']);
			nodeData['angleX'] = results;
			//console.log("angle degrees to X " + results.angleDeg);
			var prevNode = nodeData['prevNode'];
			if (prevNode) {
				var turnAngle = -nodeData['angleX'].angleDeg + prevNode['angleX'].angleDeg;
				if (Math.abs(turnAngle) > 180) {
					if (360 + turnAngle < 180)
						turnAngle = 360 + turnAngle;
					else
						turnAngle = turnAngle - 360;

				}
				results['angleTurnDeg'] = turnAngle;
				//console.log("angle turn X " + turnAngle);
			}

		}
	}

	this.calculateAngle2Vectors = function (v1, v2) {
		//angle(v1, v2) = acos( (v1x * v2x + v1y * v2y) / (sqrt(v1x^2+v1y^2) * sqrt(v2x^2+v2y^2)) )
		var angleRad = Math.acos((v1.x * v2.x + v1.y * v2.y) / (Math.sqrt(Math.pow(v1.x, 2) + Math.pow(v1.y, 2)) * Math.sqrt(Math.pow(v2.x, 2) + Math.pow(v2.y, 2))));
		var angleDeg = angleRad * 180 / Math.PI;
		var angle = {
			'angleRad' : angleRad,
			'angleDeg' : angleDeg,
		}
		return angle;

	}

	this.calculateAngle2Vectors2 = function (start, end) {
		//angle(v1, v2) = acos( (v1x * v2x + v1y * v2y) / (sqrt(v1x^2+v1y^2) * sqrt(v2x^2+v2y^2)) )

		var x = start[0] + (end[0] - start[0]) / 2;
		var y = start[1] + (end[1] - start[1]) / 2;
		//arrow.setId('arrow');
		var dx = end[0] - start[0];
		var dy = end[1] - start[1];
		var angleRad = Math.atan2(dy, dx);

		var angleDeg = angleRad * 180 / Math.PI;
		//console.log(angleDeg);
		var angleDegInverted = angleDeg * (-1)

			if (this.sign(angleDegInverted) == -1) {
				angleDegDO = 360 + angleDegInverted + 90
			} else {
				angleDegDO = angleDegInverted + 90
			}

			if (angleDegDO > 360)
				angleDegDO = angleDegDO - 360;

			//console.log(angleDegInverted + " > " + angleDegDO);

			var angle = {
			'angleRad' : angleRad,
			'angleDeg' : angleDeg,
			'angleDegDO' : angleDegDO,
		}
		return angle;

	}
	
	 this.sign = function(x) {
	  x = +x; // convert to a number
	  if (x === 0 || isNaN(x)) {
		return x;
	  }
	  return x > 0 ? 1 : -1;
	}
	
	this.userLocationChanged = function (newLocation) {
		//console.log("currentUserLocation " + newLocation);
		if (this.isActive && !this.arrived ) {
			this.updatePathDirectionsOnLocationChanged(newLocation);
			this.updatePathDirectionsOnOrientationChanged(this.navigatorCore.orientationManager.getLastKnownRotationDeg());
		}

	}

	this.deviceOrientationChanged = function (newDeviceRotation) {
		if (this.isActive)
			this.updatePathDirectionsOnOrientationChanged(newDeviceRotation);
	};

	this.updatePathDirectionsOnOrientationChanged = function (newDeviceRotation) {
		//alert ( "DirectionsOnOrientationChanged: " + newDeviceRotation);
		//document.getElementById('degrees').innerText = newDeviceRotation;
		//alert('rotation changed');
		this.deviceOrientationDirections = [];
		if (this.isOnPath) {

			this.createOrientationDirectionsOnPath(newDeviceRotation);
		} else {
			this.createOrientationDirectionsNotOnPath(newDeviceRotation);
		}
		this.updateDisplayedDirections();
	}

	
	this.getCurrentDirections = function () {
		return this.currentDirections;
	}
	
	this.updateDisplayedDirections = function () {
		//	document.getElementById('o_dir').innerText = '';
		//	document.getElementById('p_dir').innerText ='';
		$('#nav_distance').show();
		$('#nav_time').show();	
		this.currentDirections['nav_direction_curent'] = undefined;
		this.currentDirections['nav_direction_next'] = undefined;
		this.currentDirections['path_summary'] = undefined;
		
		
		$('#tour-nav-link').text(tourName);
		$("#nav_current_image").attr('class', 'no_arrow');
		$('#nav_current_text').text('');
		$("#nav_next_image").attr('class', 'no_arrow');
		$('#nav_next_text').text('');
		var hasCurrent = false;
		
			//	var s = '';
			

		if ( this.arrived){
			$("#nav_current_image").attr('class', "arrive");
			$('#nav_current_text').text(langString (language, 'direction_arrived'));
			$('#nav_distance').hide();
			$('#nav_time').hide();
			return;
		}
		
			

			if (this.deviceOrientationDirections.length > 0) {
				hasCurrent = true;
				//	document.getElementById('o_dir').innerText = JSON.stringify(this.deviceOrientationDirections[0]);
				$("#nav_current_image").attr('class', this.deviceOrientationDirections[0].type);
				$("#nav_current_image").addClass('d_' + this.deviceOrientationDirections[0].turn);
				$('#nav_current_text').text(this.deviceOrientationDirections[0].text);
				this.currentDirections['nav_direction_curent'] =this.deviceOrientationDirections[0];
			}
			//console.log(JSON.stringify(this.pathDirections))
			if (this.pathDirections.length > 0) {
			//	var s = '';
				for (var i = 0; i < this.pathDirections.length; i++) {
					var directions = this.pathDirections[i]
						//s = s + JSON.stringify(directions);

						if (!hasCurrent) {
							$("#nav_current_image").attr('class', directions.type);
							if (directions.type == "turn" ) {
								$("#nav_current_image").addClass('d_' + directions.turn);
								$('#nav_current_text').text(directions.text);
							}
							if (directions.type == "walk") {
								$('#nav_current_text').text(directions.distance + langString(language,'metres'));
							}
							if (directions.type == "arrive") {
								$('#nav_current_text').text(directions.text);
							}
							if (directions.type == "walk_to_path") {
								$('#nav_current_text').text(directions.text);
							}
							this.currentDirections['nav_direction_curent'] = directions;
							hasCurrent = true;
						
						} else {
							$("#nav_next_image").attr('class', directions.type);
							$('#nav_next_text').text(langString (language, 'direction_then'));
							if (directions.type == "turn") {
								$("#nav_next_image").addClass('d_' + directions.turn);

							}
							this.currentDirections['nav_direction_next'] = directions;
							break;
						}
				}
				//document.getElementById('p_dir').innerText =s;
			}

			// display path summary

			if (this.pathSummary) {
				$('#nav_distance').text(this.pathSummary.distanceMeters + langString(language,'metres'));
				$('#nav_time').text(this.pathSummary.timeSecondsDisplay);
				$('#nav_arrival').text(this.pathSummary.arrivalTimeDisplay);
				this.currentDirections['path_summary'] = this.pathSummary;
			}
			var lastKnownLocation = this.navigatorCore.locationManager.getLastKnownLocation();
			if (lastKnownLocation) {
			 this.currentDirections['last_known_location'] = ol.proj.toLonLat(lastKnownLocation);
			 this.currentDirections['positioning_mode'] =  this.navigatorCore.positioningMode;
			}
			
//var result = appConnector.getCurrentNavDirections();
//console.log(JSON.stringify(result));
	}

	this.createOrientationDirectionsOnPath = function (newDeviceRotation) {
		//alert('dir');
		if (this.currentClosestfeatures) {

			var nodeDataStart = this.currentClosestfeatures.link.feature.nodeDataStart;
			if (this.aproachingNode) {
				nodeDataStart = this.currentClosestfeatures.node.feature.nodeData;
			}
			var angleX = nodeDataStart['angleX'];
			//var m =this.createPathNodeFeatureHL(this.currentClosestfeatures.link.feature.nodeDataStart);
			//vectorNav.getSource().addFeature( m);
			if (angleX) {
				var angleDegDO = angleX['angleDegDO'];
				var relAngle = newDeviceRotation - angleDegDO;

				if (Math.abs(relAngle) > 180) {
					relAngle = (-1) * this.sign(relAngle) * (360 - Math.abs(relAngle));
				}

				//	document.getElementById('debug').innerText = angleDegDO;
				//	document.getElementById('debug1').innerText = relAngle;
				if (Math.abs(relAngle) > this.navigatorCore.orientationTrigger ) {
					var turnDirection = this.createTurnDirectionOrientation(-relAngle);
					this.deviceOrientationDirections.push(turnDirection);
				}
			}
		}  
	}

	this.createOrientationDirectionsNotOnPath = function (newDeviceRotation) {
		if (this.currentClosestfeatures) {
			var feature = this.currentClosestfeatures.link.feature;
			var userLocation = this.navigatorCore.locationManager.getLastKnownLocation();
			var closestPoint = feature.getGeometry().getClosestPoint(userLocation);
			if (userLocation) {
				var angleX = this.calculateAngle2Vectors2(userLocation, closestPoint);
				//var m =this.createPathNodeFeatureHL(this.currentClosestfeatures.link.feature.nodeDataStart);
				//vectorNav.getSource().addFeature( m);

				var angleDegDO = angleX['angleDegDO'];
				var relAngle = newDeviceRotation - angleDegDO;

				if (Math.abs(relAngle) > 180) {
					relAngle = (-1) * this.sign(relAngle) * (360 - Math.abs(relAngle));
				}
				//	document.getElementById('debug').innerText = angleDegDO;
				//	document.getElementById('debug1').innerText = relAngle;
				if (Math.abs(relAngle) > this.navigatorCore.orientationTrigger) {
					var turnDirection = this.createTurnDirectionOrientation(-relAngle);
					this.deviceOrientationDirections.push(turnDirection);
				}
			}
		}

	}

	this.updatePathDirectionsOnLocationChanged = function (newLocation) {
		this.pathDirections = [];
		this.pathSummary = {};
		var closestFeatures = this.getClosestPathFeatures(newLocation);
		this.currentClosestfeatures = closestFeatures

			this.isOnPath = closestFeatures.link.closetDistance * this.navigatorCore.mapMetersPerCoordinatePoint < this.navigatorCore.onPath;
		//|| closestFeatures.node.closetDistance * this.navigatorCore.mapMetersPerCoordinatePoint < this.navigatorCore.onPath

		//	console.log("isOnPath" + closestFeatures.node.closetDistance * this.navigatorCore.mapMetersPerCoordinatePoint );
		//	console.log("isOnPath" + closestFeatures.link.closetDistance * this.navigatorCore.mapMetersPerCoordinatePoint );
		if (this.isOnPath) {
			//console.log("isOnPath");
			// on path pathDirections
			var distanceToNodeAlongPath = this.getDistanceMeteres(closestFeatures.link.closestPoint, closestFeatures.node.closestPoint);
			//console.log("node dist" + closestFeatures.node.closetDistance * this.navigatorCore.mapMetersPerCoordinatePoint);
			this.aproachingNode = distanceToNodeAlongPath < this.navigatorCore.aproachingNode && closestFeatures.link.feature.nodeDataEnd === closestFeatures.node.feature.nodeData;
			//
			if (this.aproachingNode) {
				// node direction
				//console.log("nodepathDirections");
				this.updatePathDirectionsNode(closestFeatures.node.feature.nodeData);
				//console.log("near node");
				//console.log("pathDirections" + JSON.stringify(this.pathDirections));
			} else {
				// link direction
				this.updatePathDirectionsLink(closestFeatures.link.closestPoint, closestFeatures.link.feature.nodeDataEnd);
				//console.log("near link");
				//console.log("pathDirections" + JSON.stringify(this.pathDirections));
			}

		} else {
			// get to  path pathDirections
			//console.log("not on path");
			this.updatePathDirectionsNotOnPath(newLocation, closestFeatures.link.closestPoint);
			//console.log("pathDirections" + JSON.stringify(this.pathDirections));
		}
		var distance;
		if (this.isOnPath) 
		distance = closestFeatures.link.closetDistance +  this.getDistance(closestFeatures.link.closestPoint, closestFeatures.node.closestPoint);
		else 
		distance = closestFeatures.link.closetDistance +  this.getDistance(closestFeatures.link.closestPoint, closestFeatures.node.closestPoint);
		var nodeData = closestFeatures.node.feature.nodeData;
		while (nodeData.nextNode) {
			distance = distance + this.getDistance(nodeData.coord, nodeData.nextNode.coord);
			nodeData = nodeData.nextNode;
		}
		this.pathSummary.distanceMeters = Math.round(distance * this.navigatorCore.mapMetersPerCoordinatePoint);
		this.pathSummary.timeSeconds = Math.round(this.pathSummary.distanceMeters / this.navigatorCore.avarageWalkingSpeed);

		this.pathSummary.timeSecondsDisplay = displayTimeFromSecondsRounded(this.pathSummary.timeSeconds);

		//var date = new Date(null);
		//date.setSeconds(this.pathSummary.timeSeconds);
		//this.pathSummary.timeSecondsDisplay=date.toISOString().substr(11, 8);

		var arrivalDate = new Date((new Date().getTime() + 1000 * this.pathSummary.timeSeconds));
		var arrivalTimeDisplay = arrivalDate.getHours() + ":"
			 + arrivalDate.getMinutes() + ":"
			 + arrivalDate.getSeconds();

		this.pathSummary.arrivalTime = arrivalDate.getTime();
		this.pathSummary.arrivalTimeDisplay = arrivalTimeDisplay;

		if (this.pathSummary.distanceMeters <= this.navigatorCore.arrivedDistance)
			this.setArrived(true);	
		if (this.pathSummary.distanceMeters <= this.navigatorCore.approachingDistance)
			this.setApproaching(true);
		else this.setApproaching(false);
		this.updateDisplayedDirections();
		this.updateArrivedOverlay();

	}

	this.updatePathDirectionsNotOnPath = function (startPoint, pathPoint) {
		var walkingDirection = this.createWalkingDirectionWalkToPath(startPoint, pathPoint);
		this.pathDirections.push(walkingDirection);
		//add next node pathDirections
		//this.updatePathDirectionsNode(nextNode);

	}
	
	this.isTrueTurn = function (nodeData) {
		var target_id = nodeData['target_id'];
		var point = navigatorCore.navBackboneFLayer.points[target_id];
		//return point['links'] && Object.keys(point['links']).length > 2 
		var angleX = nodeData['angleX'];
		//console.log("angleX: " + angleX)
		//console.log("navigatorCore.turnInstructionCrossingMinAngle: " + navigatorCore.turnInstructionCrossingMinAngle)
		if (angleX) 	
		return(Object.keys(point['links']).length > 2 && (Math.abs(angleX['angleTurnDeg']) > navigatorCore.turnInstructionCrossingMinAngle))||((Object.keys(point['links']).length <=2 && (Math.abs(angleX['angleTurnDeg']) > navigatorCore.turnInstructionBendMinAngle)))
		else return false;
	}
	
	this.nextTrueTurn = function (nodeData) {
		
		var currentNode = nodeData;
		if (!this.isTrueTurn(currentNode)){
			while ( currentNode["nextNode"]!=undefined ) {
				if (this.isTrueTurn(currentNode["nextNode"])) return currentNode["nextNode"];
				currentNode = currentNode["nextNode"]
			}
		} else {
			return currentNode
		}

	}

	this.updatePathDirectionsLink = function (startPoint, nextNode) {
		//var walkingDirection = this.createWalkingDirection(startPoint, nextNode['coord']);
		var walkingDirection = this.createWalkingDirectionTillNextTrueTurn(startPoint, nextNode);
		this.pathDirections.push(walkingDirection);
		//add next node pathDirections
		this.updatePathDirectionsNode(nextNode);

	}

	this.updatePathDirectionsNode = function (nodeData) {

		//nodeData['coord']
		//nodeData['nav_name']
		//nodeData['type']
		//nodeData['target_id']
	//	var isTrueTurn =  this.isTrueTurn(nodeData) 
		//console.log("isTrueTurn: " + isTrueTurn)
		if( this.pathDirections.length>1) return;
		if(this.isTrueTurn(nodeData) && this.pathDirections.length>0){
			var angleX = nodeData['angleX'];
			if (angleX) {
				var angleTurnDeg = angleX['angleTurnDeg'];
				if (angleTurnDeg) {
					var turnDirection = this.createTurnDirection(angleTurnDeg);
					this.pathDirections.push(turnDirection);
				}
			}
		}
		var nextNode = nodeData['nextNode'];
		if (nextNode ) {
			//var walkingDirection = this.createWalkingDirection(nodeData['coord'], nextNode['coord']);
			if (this.aproachingNode){
				var walkingDirection = this.createWalkingDirectionTillNextTrueTurn(nodeData['coord'], nextNode);
				this.pathDirections.push(walkingDirection);
				
			}
			var nextTrueTurn = this.nextTrueTurn(nextNode)
			if (nextTrueTurn){
			//	console.log("nextTrueTurn: " +nextTrueTurn['target_id'])
				this.updatePathDirectionsNode(nextTrueTurn);
			}

		} else {
			var arriveDirection = this.createArriveDirection();
			this.pathDirections.push(arriveDirection);
		}
		var prevNode = nodeData['prevNode'];
		if (!prevNode && nextNode) {
			//add next node pathDirections
			this.updatePathDirectionsNode(nextNode);
		}

	}

	this.createArriveDirection = function (start, end) {
		var direction = {
			'type' : 'arrive',
			'text' : langString (language, 'direction_arrived')
		}
		return direction;
	}

	this.createWalkingDirection = function (start, end) {
		var distance = this.getDistanceMeteres(start, end);
		var direction = {
			'type' : 'walk',
			'distance' : distance,
			'text' : langString (language, 'direction_walk')
		}
		return direction;
	}
	
	this.createWalkingDirectionTillNextTrueTurn = function (start, startNodeData) {
		var distance = this.getDistanceMeteres(start, startNodeData['coord']);
		var currentNode = startNodeData;
		if (!this.isTrueTurn(currentNode)){
			while ( currentNode["nextNode"]!=undefined ) {
				distance += this.getDistanceMeteres(currentNode['coord'], currentNode["nextNode"]['coord']);
				if (this.isTrueTurn(currentNode["nextNode"])) break;
				currentNode = currentNode["nextNode"]
			}
		}
		var direction = {
			'type' : 'walk',
			'distance' : distance,
			'text' : langString (language, 'direction_walk')
		}
		return direction;
	}
	
	this.createWalkingDirectionWalkToPath = function (start, end) {
		var distance = this.getDistanceMeteres(start, end);
		var direction = {
			'type' : 'walk_to_path',
			'distance' : distance,
			'text' : langString (language, 'direction_walk_to_path')

		}
		return direction;
	}

	this.createTurnDirection = function (angleTurnDeg) {
		var direction = {
			'type' : 'turn',
			'deeres' : angleTurnDeg
		}

		if (Math.abs(angleTurnDeg) > 160) {
			direction.turn = 'turn_around';
			direction.text = langString (language, 'direction_turn_around');
		} else {
			if (this.sign(angleTurnDeg) == 1) {
				if (Math.abs(angleTurnDeg) < 45) {
					direction.turn = 'slight_right';
					direction.text = langString (language, 'direction_slight_right');
				} else if (Math.abs(angleTurnDeg) >= 45 && Math.abs(angleTurnDeg) <= 135) {
					direction.turn = 'right';
					direction.text = langString (language, 'direction_right');
				} else {
					direction.turn = 'sharp_right';
					direction.text = langString (language, 'direction_sharp_right');
				}

			} else {
				if (Math.abs(angleTurnDeg) < 45) {
					direction.turn = 'slight_left';
					direction.text = langString (language, 'direction_slight_left');
				} else if (Math.abs(angleTurnDeg) >= 45 && Math.abs(angleTurnDeg) <= 135) {
					direction.turn = 'left';
					direction.text = langString (language, 'direction_left');
				} else {
					direction.turn = 'sharp_left';
					direction.text = langString (language, 'direction_sharp_left');
				}

			}

		}

		return direction;

	}

	this.createTurnDirectionOrientation = function (angleTurnDeg) {
		var direction = {
			'type' : 'turn',
			'deeres' : angleTurnDeg
		}

		if (Math.abs(angleTurnDeg) > 135) {
			direction.turn = 'turn_around';
			direction.text = langString (language, 'direction_turn_around');
		} else {
			if (this.sign(angleTurnDeg) == 1) {
				if (Math.abs(angleTurnDeg) < 45) {
					direction.turn = 'slight_right';
					direction.text = langString (language, 'direction_slight_right');
				} else if (Math.abs(angleTurnDeg) >= 45 && Math.abs(angleTurnDeg) <= 135) {
					direction.turn = 'right';
					direction.text = langString (language, 'direction_right');
				}

			} else {
				if (Math.abs(angleTurnDeg) < 45) {
					direction.turn = 'slight_left';
					direction.text = langString (language, 'direction_slight_left');
				} else if (Math.abs(angleTurnDeg) >= 45 && Math.abs(angleTurnDeg) <= 135) {
					direction.turn = 'left';
					direction.text = langString (language, 'direction_left');
				}

			}

		}

		return direction;

	}
	
	this.getClosestPathFeatures = function (newLocation) {
		var dest = this.pathFeatures[this.pathFeatures.length-1].getGeometry().getCoordinates();
		var result = {};
		var closestNode = {};
		var closestLink = {};
		var closestObject;

		for (var i = 0; i < this.pathFeatures.length; i++) {
			var feature = this.pathFeatures[i]
				if (feature.get('type') == 'path-link') {
					closestObject = closestLink;
				if (!closestObject.closetDistance) {
					closestObject.closestPoint = feature.getGeometry().getClosestPoint(newLocation);
					closestObject.closetDistance = this.navigatorCore.navBackboneFLayer.getDistance(newLocation, closestObject.closestPoint);
					closestObject.feature = feature;
				} else {
					var newClosestPoint = feature.getGeometry().getClosestPoint(newLocation);
					var newClosestDistance = this.navigatorCore.navBackboneFLayer.getDistance(newLocation, newClosestPoint);
					var newDistanceToDestination = this.navigatorCore.navBackboneFLayer.getDistance( newClosestPoint,dest);
					if (newClosestDistance < closestObject.closetDistance) {
						closestObject.closestPoint = newClosestPoint;
						closestObject.closetDistance = newClosestDistance;
						closestObject.feature = feature;
					}
				}
			}

		}
		
		var end2 = closestLink.feature['nodeDataEnd']['feature']
		closestNode = {};
		closestNode.feature = end2;
		closestNode.closestPoint = end2.getGeometry().getClosestPoint(newLocation);
		result['node'] = closestNode;
		result['link'] = closestLink;
		//	console.log("closest node= " +  result['node'].closestPoint);
		//	console.log("closest link start= " +  result['link'].feature.nodeDataStart["target_id"]);
		//	console.log("closest link end " +  result['link'].feature.nodeDataEnd["target_id"]);

		return result;
	}

	this.getDistanceMeteres = function (coord1, coord2) {
		return Math.round(Math.sqrt(Math.pow(coord1[0] - coord2[0], 2) + Math.pow(coord1[1] - coord2[1], 2)) * navigatorCore.mapMetersPerCoordinatePoint);
	}

	this.getDistance = function (coord1, coord2) {
		return Math.round(Math.sqrt(Math.pow(coord1[0] - coord2[0], 2) + Math.pow(coord1[1] - coord2[1], 2)));
	}

}

function TourPathAssistant(navigatorCore, vectorNav) {
	this.vectorNav = vectorNav;
	this.navigatorCore = navigatorCore;
	this.isActive = false;
	this.pathFeatures = [];
	this.pathDirections = [];
	this.deviceOrientationDirections = [];
	this.currentDirections = {};
	
	
	this.displayNavPath = function (shortestPath) {

		var pathCoord = [];

		var startLocation = shortestPath['startLocation'];
		if (startLocation) {
			pathCoord.push(startLocation['coord']);
		}

		var navPath = shortestPath['routingData']['p'];
		for (var i = 0; i < navPath.length; i++) {
			var id = navPath[i];
			var node = this.navigatorCore.navBackboneFLayer.points[id];
			pathCoord.push(node['coord']);
		}

		var destinationLocation = shortestPath['destinationLocation'];
		if (destinationLocation) {
			pathCoord.push(destinationLocation['coord']);
		}

		//	if (destinationLocation && destinationLocation['itemCoord']) {
		//	pathCoord.push(destinationLocation['itemCoord']);
		//	}

		var navPathFeature = this.createNavPathFeature(pathCoord);
	//	var navPathFeatureCore = this.createNavPathFeatureCore(pathCoord);

		this.vectorNav.getSource().addFeature(navPathFeature);
	//	this.vectorNav.getSource().addFeature(navPathFeatureCore);
		//this.addArrowsToNavPath(navPathFeature);
	}

	this.clearTourPathsCache = function () {
		var features = this.vectorNav.getSource().getFeatures();
		for (var i = 0; i < features.length; i++) {
			if (features[i].get('purpose')=='tourPath')
			this.vectorNav.getSource().removeFeature(features[i]);
		}

	}

	this.navPathStyle = new ol.style.Style({ //  My style definition
			fill : new ol.style.Fill({
				color : 'rgba(0, 0, 255, 0.5)',
			}),
			stroke : new ol.style.Stroke({
				color : 'rgba(0, 0, 255, 0.5)',
				width : 10
				//lineCap: 'square',
				// lineDash: [20,40] // <-Here is the change. first line then space. try [40,40] if not shown as dash
			})
		});

	this.createNavPathFeature = function (pathCoord) {
		var linkFeature =
			new ol.Feature({
				id : 'linkFeature',
				type : 'nav-link-path',
				purpose: 'tourPath',
				geometry : new ol.geom.LineString(pathCoord),
				name : 'link'
			});
		//nodeFeature.setId(id);
		linkFeature.setStyle(this.navPathStyle);
		
		//nodeFeature.setId(id);
		return linkFeature;

	}

	this.navPathCoreStyle = new ol.style.Style({ //  My style definition
			fill : new ol.style.Fill({
				//color : 'rgba(255, 204, 51, 1)'
				color : 'rgba(255, 255, 255, 1)'
			}),
			stroke : new ol.style.Stroke({
				//color : 'rgba(255, 204, 51, 1)',
				color : 'rgba(255, 255, 255, 1)',
				width : 2

				// lineCap: 'square',
				// lineDash: [20,40] // <-Here is the change. first line then space. try [40,40] if not shown as dash
			})
		});

	this.createNavPathFeatureCore = function (pathCoord) {
		var linkFeature =
			new ol.Feature({
				id : 'linkFeature',
				type : 'nav-link-core',
				purpose: 'tourPath',
				geometry : new ol.geom.LineString(pathCoord),
				name : 'link'
			});
		//nodeFeature.setId(id);
		linkFeature.setStyle(this.navPathCoreStyle);
		//nodeFeature.setId(id);
		return linkFeature;

	}
}